import { BASEURL, PERMISSIONNAME } from './config';

/********************************************
 *             系统和业务                   *
 ******************************************/

// 本地存储
const storage = {
  set(key, value) {
    api.setPrefs({
      key,
      value: typeof value === 'string' ? value : JSON.stringify(value),
    });
  },

  get(key) {
    const value = api.getPrefs({ sync: true, key }) || '';

    try {
      return JSON.parse(value);
    } catch (e) {
      return value;
    }
  },

  del(key) {
    api.removePrefs({ key });
  },
};

// 请求
function ajax({ url, method = 'get', data = {} }) {
  return new Promise((resolve, reject) => {
    api.ajax(
      {
        url: url.indexOf('http') === 0 ? url : BASEURL + url,
        method,
        headers: {
          'Content-Type': 'application/json;charset=utf-8',
          'ZK-TOKEN': storage.get('token'),
          'APP-TYPE': 2
        },
        timeout: 60,
        data: {
          body: data,
        },
      },
      (ret, err) => {
        if (ret) {
          if (ret.code === 200) {
            resolve(ret.data);
          } else if (ret.code === 600) {
            api.sendEvent({
              name: 'loginExpired',
            });
          } else {
            api.toast({
              msg: ret.msg,
            });
          }

          reject(ret);
        } else {
          api.toast({
            msg: '网络不畅,请检查网络',
          });
          reject(err);
        }
      },
    );
  });
}

// 获取登录状态
function getLoginStatus() {
  return storage.get('isLogin') || false;
}

/**
 * 设置登录状态
 * @param {Boolean} status 登录状态
 */
function setLoginStatus(status) {
  storage.set('isLogin', !!status);
}

// 获取用户信息（包括昵称、头像、个性签名等）
function getUserInfo() {
  return storage.get('userInfo') || {};
}

/**
 * 保存用户信息（为方便获取token，这里有意分开保存userInfo和token）
 * @param {Object} userInfo 用户信息
 * @param {String} token 用户TOKEN
 */
function saveUserInfo(userInfo, token) {
  storage.set('userInfo', userInfo);
  if (token) {
    storage.set('token', token);
  }
}

/**
 * 更新用户信息
 * @param {String} key 修改的字段
 * @param {String} value 修改后的值
 */
function updateUserInfo(key, value) {
  // 更新字段
  const userInfo = getUserInfo();
  userInfo[key] = value;

  // 保存用户信息
  saveUserInfo(userInfo);

  // 发送更新用户信息事件
  api.sendEvent({
    name: 'updateUserInfo',
  });
}


// 清空用户信息
function clearUserInfo() {
  storage.del('token');
  storage.del('userInfo');
}

// 打开登录页面
function openLogin() {
  api.openTabLayout({
    name: 'login',
    url: '../login/login.stml',
    navigationBar: {
      background: '#5538de',
      shadow: '#5538de',
      color: '#fff',
    },
  });
}

/**
 * 用户登录成功
 * @param {Object} userInfo 用户信息
 */
function loginSuccess({
  userId, phone, token, nick, photo, signature, userDeviceCodes,userLock
}) {
  // 设置用户登录状态
  setLoginStatus(true);

  // 保存用户信息
  saveUserInfo(
    {
      userId,
      phone,
      nickname: nick || '愤斗',
      avatarUrl: photo || 'https://files.zhuokao360.com/user/photo/avatar@default.png',
      info: signature || '这家伙很懒，什么都没有留下。',
      userDeviceCodes,
      userLock

    },
    token,
  );

  closeToRoot();
  // 发送登录事件
  api.sendEvent({
    name: 'login',
  });
}

// 保存上次登录的账号
function saveLastLogin() {
  const { phone } = getUserInfo();
  storage.set('lastLogin', phone);
}

// 获取上次登录的账号
function getLastLogin() {
  return storage.get('lastLogin');
}

// 用户退出
function exit() {
  setLoginStatus(false);
  saveLastLogin();
  clearUserInfo();

  closeToRoot();

  // 发送退出事件
  api.sendEvent({
    name: 'exitStatus',
  });
}


/**
 * 判断是否拥有设备权限
 * @param {Array} perms 权限列表
 */
function hasPermission(perms) {
  const resultList = api.hasPermission({
    list: perms,
  });
  return resultList;
}

/**
 * 向系统请求设备权限
 * @param {Array} perms 权限列表
 */
function requestPermission(perms) {
  api.requestPermission({
    list: perms,
    code: 1,
  });
}

/**
 * 确认设备权限(无权限提示获取)
 * @param {Array} perms 权限列表
 */
function confirmPermission(perms) {
  const resultList = hasPermission(perms);

  // 获取需要请求的权限列表
  const requestList = [];
  resultList.forEach((item, index) => {
    if (!item || !item.granted) {
      requestList.push(perms[index]);
    }
  });

  // 如果拥有全部权限
  if (requestList.length === 0) {
    return true;
  }

  // 否则，提示请求权限
  let str = '';
  requestList.forEach((per) => {
    str += `${PERMISSIONNAME[per]}、`;
  });
  str = str.slice(0, str.length - 1);

  if (str === '存储') {
    api.confirm(
      {
        title: '权限说明',
        msg: `应用对${str}权限申请说明：便于您使用该功能保存文件/视频用于缓存离线视频、讲义的场景中读取和写入设备存储。`,
        buttons: ['去设置', '取消'],
      },
      (ret) => {
        if (ret.buttonIndex === 1) {
          requestPermission(perms);
        }
      },
    );
  } else {
    api.confirm(
      {
        title: '权限说明',
        msg: `应用对${str}权限申请说明：便于您使用该功能上传照片/图片用于意见反馈、保存相册、下载的场景中读取和写入相册。`,
        buttons: ['去设置', '取消'],
      },
      (ret) => {
        if (ret.buttonIndex === 1) {
          requestPermission(perms);
        }
      },
    );
  }

  return false;
}

/**
 * 关闭窗口
 * @param {String} name window 名字
 * @param {Number} timeout 延迟时间
 */
function closeWin({ name, timeout } = {}) {
  function close() {
    if (typeof name === 'string') {
      api.closeWin({ name });
    } else {
      api.closeWin();
    }
  }

  if (typeof timeout === 'number') {
    setTimeout(close, timeout);
  } else {
    close();
  }
}

/**
 * 关闭到根页面
 */
function closeToRoot() {
  api.closeToWin({
    name: 'root',
  });

  // 始终竖屏显示
  api.setScreenOrientation({
    orientation: 'portrait_up'
  });
}


/**
 * 格式化文件大小, 输出成带单位的字符串
 * @param {Number} size 文件大小
 */
function formatFileSize(size) {
  const units = ['B', 'K', 'M', 'G', 'TB'];
  let fileSize = parseInt(size);
  for (let i = 0; fileSize >= 1024; i += 1) {
    fileSize /= 1024;
    units.shift();
  }
  return fileSize.toFixed(1) + units[0];
}

/**
 * 校验手机号
 * @param {Number} phone 手机号
 */
function validatorPhone(phone) {
  return /^1[3456789]\d{9}$/.test(phone);
}





/**
 * 浏览器打开
 * @param {String} url 需要打开的地址
 */
function openBrowser(url) {
  // api.openApp({
  //   androidPkg: 'android.intent.action.VIEW',
  //   uri: url,
  //   iosUrl: url,
  // });
  // ios
  if (api.systemType == 'ios') {
      api.openApp({
          iosUrl: url,
       });
  }else{
    // 安卓
  
    var jumpbrowser =api.require('jumpbrowser')
    let param = {Url:url};
    jumpbrowser.jumpBrowser(param, function(ret, err){
    if (err) {
      api.alert({
        title: '提示',
        msg: '出错了,请稍后再试',
      })
    }
  });
  }

  

}

/**
 * 未登录做题提示
 */
function noLoginDoQuestionTip() {
  api.confirm({
    title: '需要登录',
    msg: '为了保存您的做题进度，请登录后使用。',
    buttons: ['去登录', '稍后再说']
  }, ret => {
    if (ret.buttonIndex === 1) {
      openLogin();
    }
  });
}

/**
 * 打开一个带导航的本地页面
 * @param {String} name 标题
 * @param {String} title 标题
 * @param {String} url 网络地址
 */
function openNavStml(name, title, url) {
  api.openTabLayout({
    name,
    title,
    url,
    navigationBar: {},
  });
}

/**
 * 打开试听课程页面
 * @param {String} name 标题
 * @param {String} title 标题
 * @param {String} url 网络地址
 */
function openFreePage() {
  openNavStml('free', '试听课程', './free.stml')
}

/**
 * 打开一个webview页面
 * @param {String} title 标题
 * @param {String} url 网络地址
 */
function openWebview(title, url) {
  api.openTabLayout({
    name: 'webview',
    url,
    title,
    navigationBar: {},
    slidBackEnabled: false, // 是否支持滑动返回
    avm: false
  });
}

// 判断是否首次启动APP
function isFirstLaunch() {
  const isFirst = storage.get('isFirstLaunch');
  return typeof isFirst === 'boolean' ? isFirst : true;
}

// 设置非首次启动
function setNonFirstLaunch() {
  return storage.set('isFirstLaunch', false);
}

// 获取当前网络状态
function getNetworkStatus() {
  return api.connectionType !== 'none';
}

function toast(msg) {
  api.toast({ msg })
}

function log(msg) {
  if (typeof msg === "object") {
    console.log(JSON.stringify(msg));
  } else {
    console.log(msg);
  }
}

function switchTab(name, url) {
  api.openWin({ name, url });
}

/********************************************
 *             APICLOUD                   *
 ******************************************/

export {
  storage,
  ajax,
  formatFileSize,
  closeWin,
  openBrowser,
  openWebview,
  isFirstLaunch,
  setNonFirstLaunch,
  loginSuccess,
  openNavStml,
  openFreePage,
  noLoginDoQuestionTip,
  openLogin,
  exit,
  getUserInfo,
  getLastLogin,
  getLoginStatus,
  closeToRoot,
  confirmPermission,
  getNetworkStatus,
  validatorPhone,
  toast,
  log,
  switchTab,
};