import * as cedar from './cedar';

/**
 * 自动登录
 */
function autoLogin() {
  return cedar.ajax({
    url: 'login/autoLogin',
    data: {
      appType: 4
    },
  });
}

/**
 * 账户密码登录
 * @param {String} phone 手机号
 * @param {String} password 密码
 */
function login(phone, password) {
  return cedar.ajax({
    url: 'login/login',
    method: 'post',
    data: {
      phone,
      password,
      appType: 4
    },
  });
}

/**
 * 短信前置-获取图片验证码
 * @param {String} phone 手机号码
 */
function getImageCode(phone) {
  return cedar.ajax({
    url: 'sms/getImageCode',
    data: {
      phone,
      appType: 4
    },
  });
}

/**
 * 获取短信验证码
 * @param {String} phone 手机号码
 * @param {Number} imageCode 4位数字的图片验证码
 */
function sendSmscode(phone, imageCode) {
  return cedar.ajax({
    url: 'sms/sendSms',
    method: 'post',
    data: {
      phone,
      imageCode,
      type: 34,
      appType: 4
    },
  });
}

/**
 * 短信验证码登录
 * @param {String} phone 手机号码
 * @param {Number} smscode 6位数字的短信验证码
 */
function smscodeLogin(phone, smscode) {
  return cedar.ajax({
    url: 'login/sms',
    method: 'post',
    data: {
      phone,
      smsCode: smscode,
      type: 34,
      appType: 4
    },
  });
}

/**
 * 修改密码
 * @param {String} oldPwd 原密码
 * @param {String} newPwd 新密码
 */
function changePassword(oldPwd, newPwd) {
  return cedar.ajax({
    url: 'user/changePassword',
    data: {
      oldPwd,
      newPwd
    },
  });
}

/**
 * 获取登录状态
 */
function getLoginStatus() {
  return cedar.storage.get('isLogin') || false;
}

/**
 * 设置登录状态
 * @param {Boolean} status 登录状态
 */
function setLoginStatus(status) {
  cedar.storage.set('isLogin', !!status);
}

/**
 * 获取用户信息（包括昵称、头像、个性签名等）
 */
function getUserInfo() {
  return cedar.storage.get('userInfo') || {};
}

/**
 * 保存用户信息（为方便获取token，这里有意分开保存userInfo和token）
 * @param {Object} userInfo 用户信息
 * @param {String} token 用户TOKEN
 */
function saveUserInfo(userInfo, token) {
  cedar.storage.set('userInfo', userInfo);
  if (token) {
    cedar.storage.set('token', token);
  }
}

/**
 * 更新用户信息
 * @param {String} key 修改的字段
 * @param {String} value 修改后的值
 */
function updateUserInfo(key, value) {
  // 更新字段
  const userInfo = getUserInfo();
  userInfo[key] = value;

  // 保存用户信息
  saveUserInfo(userInfo);

  // 发送更新用户信息事件
  api.sendEvent({
    name: 'updateUserInfo',
  });
}


/**
 * 清空用户信息
 */
function clearUserInfo() {
  cedar.storage.del('token');
  cedar.storage.del('userInfo');
}

/**
 * 打开登录页面
 */
function openLogin() {
  api.openTabLayout({
    name: 'login',
    url: '../login/login.stml',
    navigationBar: {
      background: '#5538de',
      shadow: '#5538de',
      color: '#fff',
    },
  });
}

/**
 * 用户登录成功
 * @param {Object} userInfo 用户信息
 */
function loginSuccess({
  userId, phone, token, nickname, avatarUrl, userDeviceCodes,
}) {
  // 设置用户登录状态
  setLoginStatus(true);

  // 保存用户信息
  saveUserInfo(
    {
      userId,
      phone,
      nickname: nickname || '同学',
      avatarUrl: avatarUrl || '../../image/default-avatar.png',
      userDeviceCodes: userDeviceCodes,
    },
    token,
  );

  cedar.closeToRoot();
  // 发送登录事件
  api.sendEvent({
    name: 'login',
  });
}

/**
 * 保存上次登录的账号
 */
function saveLastLogin() {
  const { phone } = getUserInfo();
  cedar.storage.set('lastLogin', phone);
}

/**
 * 获取上次登录的账号
 */
function getLastLogin() {
  return cedar.storage.get('lastLogin');
}

/**
 * 用户退出
 */
function exit() {
  setLoginStatus(false);
  saveLastLogin();
  clearUserInfo();

  cedar.closeToRoot();

  // 发送退出事件
  api.sendEvent({
    name: 'exitStatus',
  });
}


/************************************************
*                   课程接口                    *      
***********************************************/

/**
 * 获取首页公告提示
 */
function getNotice(type, professionId) {
  return cedar.ajax({
    url: `notice/findNotice/${type}/${professionId}`,
  });
}

/**
 * 获取所有课程
 */
function getAllCourse() {
  return cedar.ajax({
    url: 'course/findAllCourseAuth',
    data: {
      appType: 4,
    }
  });
}

/**
 * 获取课程列表
 */
function getCourseList() {
  return cedar.ajax({
    url: 'course/findCourseAuth',
    data: {
      appType: 4,
    }
  });
}

/**
 * 获取课程统计数据
 * @param {Number} courseId 课程id
 * @param {Number} type 课程类型
 */
function getCourseStats(courseId, type = 1) {
  return cedar.ajax({
    url: 'courseStage/getCourseLearnInfo',
    data: {
      courseId,
      type,
      appType: 4
    },
  });
}

/**
 * 获取课时列表
 * @param {Number} courseId 课程id
 * @param {Number} type 课程类型
 */
function getLessonList(courseId, type = 1) {
  return cedar.ajax({
    url: 'courseStage/findCourseStageByType',
    data: {
      courseId,
      type,
      appType: 4
    },
  });
}

/**
 * 获取真题列表
 * @param {Number} courseId 课程id
 */
function getExamList(courseId) {
  return cedar.ajax({
    url: 'section/findSectionBy',
    data: {
      courseId,
      sectionType: 8,
      appType: 4
    },
  });
}

/**
 * 是否有资料
 * @param {Number} courseId 课程id
 * @param {Number} type 课程类型
 */
function isHasFile(courseId, type) {
  return cedar.ajax({
    url: 'videoHandout/isHas',
    data: {
      courseId,
      type,
      appType: 4
    },
  });
}

/**
 * 获取资料列表，读取讲义表
 * @param {Number} courseId 课程id
 * @param {Number} type 讲义类型
 */
function getFileList(courseId, type) {
  return cedar.ajax({
    url: 'videoHandout/findVideoHandout',
    data: {
      courseId,
      type,
      appType: 4
    },
  });
}

/**
 * 获取某一课时信息
 * @param {Number} courseStageId 课时id
 */
function getCourseDetail(courseStageId) {
  return cedar.ajax({
    url: 'courseStage/getStageVideo',
    data: {
      courseStageId,
      appType: 4
    },
  });
}

/**
 * 获取视频详细信息(未登录)
 * @param {Number} videoId 视频id
 */
function getVideoDetailNo(videoId) {
  return cedar.ajax({
    url: 'videoPlus/getVideo',
    data: {
      videoId,
      appType: 4
    },
  });
}

/**
 * 获取视频详细信息
 * @param {Number} videoId 视频id
 */
function getVideoDetail(videoId) {
  return cedar.ajax({
    url: 'video/getDetail',
    data: {
      videoId,
      appType: 4,
      edition: 'v1_0_14'
    },
  });
}

/**
 * 获取加密视频信息（暂未启用）
 * @param {Number} videoId 视频id
 * @param {String} channelFlag 访问来源
 * @param {String} resourcesFlag 资源种类
 */
function getVideoInfoByAuth(videoId) {
  return cedar.ajax({
    url: 'videoPlus/getPlayInfo',
    data: {
      videoId,
      channelFlag: 1,
      resourcesFlag: 1
    },
  });
}

/**
 * 获取视频讲义
 * @param {Number} videoId 视频id
 */
function getVideoHandout(videoId) {
  return cedar.ajax({
    url: 'courseStage/getVideoHandoutByVideoId',
    data: {
      videoId,
      appType: 4
    },
  });
}

/**
 * 保存用户观看视频进度
 * @param {Number} videoId 视频id
 * @param {Number} watchDuration 视频观看时长
 */
function saveVideoProgress(videoId, watchDuration) {
  return cedar.ajax({
    url: 'video/updateWatched',
    method: 'post',
    data: {
      videoId,
      watchDuration,
      appType: 4
    },
  });
}


export {
  autoLogin,
  login,
  getLoginStatus,
  setLoginStatus,
  getUserInfo,
  saveUserInfo,
  updateUserInfo,
  openLogin,
  loginSuccess,
  exit,
  getImageCode,
  sendSmscode,
  smscodeLogin,
  getLastLogin,
  getNotice,
  getAllCourse,
  getCourseList,
  getLessonList,
  getCourseStats,
  getExamList,
  isHasFile,
  getFileList,
  getCourseDetail,
  getVideoDetailNo,
  getVideoDetail,
  getVideoInfoByAuth,
  getVideoHandout,
  saveVideoProgress,
  changePassword,
};
