import { BASEURL, PERMISSIONNAME } from './config';

// 本地存储
const storage = {
  set(key, value) {
    api.setPrefs({
      key,
      value: typeof value === 'string' ? value : JSON.stringify(value),
    });
  },

  get(key) {
    const value = api.getPrefs({ sync: true, key }) || '';

    // 如果JSON字符串无法解析，直接返回
    try {
      return JSON.parse(value);
    } catch (e) {
      return value;
    }
  },

  del(key) {
    api.removePrefs({ key });
  },
};

// 全局数据
const memory = {
  set(key, value) {
    api.setGlobalData({ key, value });
  },

  get(key) {
    return api.getGlobalData({ key });
  },
};

// 请求
function ajax({ url, method = 'get', data = {} }) {
  return new Promise((resolve, reject) => {
    api.ajax(
      {
        url: url.indexOf('http') === 0 ? url : BASEURL + url,
        method,
        headers: {
          'Content-Type': 'application/json;charset=utf-8',
          'ZK-TOKEN': storage.get('token'),
          'APP-TYPE': 2
        },
        timeout: 60,
        data: {
          body: data,
        },
      },
      (ret, err) => {
        if (ret) {
          if (ret.code === 200) {
            resolve(ret.data);
          } else if (ret.code === 600) {
            // 发送用户登录过期事件
            api.sendEvent({
              name: 'loginExpired',
            });
          } else {
            api.toast({
              msg: ret.msg,
            });
          }

          reject(ret);
        } else {
          api.toast({
            msg: '网络不畅,请检查网络',
          });
          reject(err);
        }
      },
    );
  });
}

/**
 * 判断是否拥有权限
 * @param {Array} perms 权限列表
 */
function hasPermission(perms) {
  const resultList = api.hasPermission({
    list: perms,
  });
  return resultList;
}

/**
 * 向系统请求权限
 * @param {Array} perms 权限列表
 */
function requestPermission(perms) {
  api.requestPermission({
    list: perms,
    code: 1,
  });
}

/**
 * 确认权限(无权限提示获取)
 * @param {Array} perms 权限列表
 */
function confirmPermission(perms) {
  const resultList = hasPermission(perms);

  // 获取需要请求的权限列表
  const requestList = [];
  resultList.forEach((item, index) => {
    if (!item || !item.granted) {
      requestList.push(perms[index]);
    }
  });

  // 如果拥有全部权限
  if (requestList.length === 0) {
    return true;
  }

  // 否则，提示请求权限
  let str = '';
  requestList.forEach((per) => {
    str += `${PERMISSIONNAME[per]}、`;
  });
  str = str.slice(0, str.length - 1);

  api.confirm(
    {
      title: '提醒',
      msg: `使用相关功能，需要获取 ${str} 权限\n是否前往设置？`,
      buttons: ['去设置', '取消'],
    },
    (ret) => {
      if (ret.buttonIndex === 1) {
        requestPermission(perms);
      }
    },
  );
  return false;
}

/**
 * 关闭窗口
 * @param {String} name window 名字
 * @param {Number} timeout 延迟时间
 */
function closeWin({ name, timeout } = {}) {
  function close() {
    if (typeof name === 'string') {
      api.closeWin({ name });
    } else {
      api.closeWin();
    }
  }

  if (typeof timeout === 'number') {
    setTimeout(close, timeout);
  } else {
    close();
  }
}

// 关闭到根页面
function closeToRoot() {
  api.closeToWin({
    name: 'root',
  });

  // 始终竖屏显示
  api.setScreenOrientation({
    orientation: 'portrait_up'
  });
}

/**
 * 格式化文件大小, 输出成带单位的字符串
 * @param {Number} size 文件大小
 */
function formatFileSize(size) {
  const units = ['B', 'K', 'M', 'G', 'TB'];
  let fileSize = parseInt(size);
  for (let i = 0; fileSize >= 1024; i += 1) {
    fileSize /= 1024;
    units.shift();
  }
  return fileSize.toFixed(1) + units[0];
}

/**
 * 浏览器打开
 * @param {String} url 需要打开的地址
 */
function openBrowser(url) {
  api.openApp({
    androidPkg: 'android.intent.action.VIEW',
    uri: url,
    iosUrl: url,
  });
}

/**
 * 打开一个webview页面
 * @param {String} title 标题
 * @param {String} url 网络地址
 */
function openWebview(title, url) {
  api.openTabLayout({
    name: 'webview',
    url,
    title,
    navigationBar: {},
    slidBackEnabled: false, // 是否支持滑动返回
    avm: false
  });
}

// 判断是否首次启动APP
function isFirstLaunch() {
  const isFirst = storage.get('isFirstLaunch');
  return typeof isFirst === 'boolean' ? isFirst : true;
}

// 设置非首次启动
function setNonFirstLaunch() {
  return storage.set('isFirstLaunch', false);
}

// 获取当前网络状态
function getNetworkStatus() {
  return api.connectionType !== 'none';
}

function log(msg) {
  if (typeof msg === "object") {
    console.log(JSON.stringify(msg));
  } else {
    console.log(msg);
  }
}

function toast(msg) {
  api.toast({
    msg,
  })
}

export {
  storage,
  memory,
  ajax,
  formatFileSize,
  closeWin,
  openBrowser,
  openWebview,
  isFirstLaunch,
  setNonFirstLaunch,
  closeToRoot,
  confirmPermission,
  getNetworkStatus,
  log,
  toast
};