/*
 * 播放器模块，封装了视频播放相关功能
 */

export default {
  module: null, // 播放器模块
  positionIntervId: null, // 报告当前播放位置的计时器id
  positionDelay: 30 * 1000, // 30s报告一次

  // 初始化
  init() {
    this.module = api.require('playModuleAli'); // 引入模块

    // 初始化播放器
    this.module.init({ FullScreenViewIsFont: false, isShowBottomBtn: true }, (ret, err) => {
      console.log(`${new Date()}初始化播放器：${JSON.stringify(ret)}`);
    });

    this.initEventListener();
  },

  // 首次播放
  play({
    x = 0, y = 0, w = 320, h = 250, title, url, position,
  }) {
    this.module.play(
      {
        rect: {
          x,
          y,
          w,
          h,
        },
        title, // 标题
        url,
        process: position, // 播放进度位置
        isShowSpeed: true,
        speedDatas: [
          {
            name: 'X0.75',
            speed: 0.75,
          },
          {
            name: 'X1.0',
            speed: 1.0,
          },
          {
            name: 'X1.25',
            speed: 1.25,
          },
          {
            name: 'X1.5',
            speed: 1.5,
          },
          {
            name: 'X2.0',
            speed: 2.0,
          },
        ],
        isBackBtn: true, // 小窗口是否显示返回按钮
        isLongShowBackBtn: true, // 窗口模式下是否长时间显示返回按钮
        isOpenDanmu: false, // 是否打开弹幕功能
        singlePageModule: true, // 是否打开单页面播放模式
        scalingMode: 2, // 视频播放显示铺满模式（2:适应大小模式）
        options: { "Format": { "dns_cache_clear": 1 } }, // 关闭播放器的dns缓存
      },
      (ret, err) => {
        console.log(`${new Date()}播放视频:${JSON.stringify(ret)}`);
      },
    );
  },

  // 暂停
  pause() {
    this.module.pause();
  },

  // 设置播放进度位置
  seekTo(position) {
    if (!position) {
      return;
    }

    this.module.seekTo({
      process: position
    });
  },

  // 获取当前播放位置
  getCurrentPosition() {
    return new Promise((resolve) => {
      this.module.getCurrentPosition((ret, err) => {
        if (ret && ret.status) {
          resolve(ret.currentPosition);
        }
      });
    });
  },

  // 获取视频的时长
  getDuration() {
    return new Promise((resolve) => {
      this.module.getDuration((ret, err) => {
        if (ret && ret.status) {
          resolve(ret.duration);
        }
      });
    });
  },

  // 清除页面所有打开的播放器
  cleanPlayers() {
    this.module.cleanPlayers();
    this.module.stop();
  },

  // 退出全屏
  unfull() {
    this.module.unfull();
  },

  // 关闭播放器及页面
  close() {
    this.stopReportPosition();
    this.cleanPlayers();
    api.closeWin();
  },

  // 报告当前播放位置
  reportCurrentPosition() {
    Promise.all([this.getCurrentPosition(), this.getDuration()]).then((res) => {
      const [position, duration] = res;
      if (position > 0 && duration > 0 && position <= duration) {
        api.sendEvent({
          name: 'currentPosition',
          extra: {
            position: Math.round(position / 1000),
            duration: Math.round(duration / 1000),
          },
        });
      }
    });
  },

  // 启动报告当前播放位置定时器
  startReportPosition() {
    this.stopReportPosition();

    this.positionIntervId = setInterval(() => {
      this.reportCurrentPosition();
    }, this.positionDelay);
  },

  // 停止报告当前播放位置
  stopReportPosition() {
    if (this.positionIntervId) {
      clearInterval(this.positionIntervId);
      this.positionIntervId = null;

      // 最后报告一次当前播放位置
      this.reportCurrentPosition();
    }
  },

  // 初始化播放器监听事件
  initEventListener() {
    // 监听设备 back 键被点击事件（仅 Android 平台有效）
    api.addEventListener({
        name: 'keyback',
      }, () => {
        this.module.isFullScreen((ret, err) => {
          // 全屏下退出全屏，非全屏下关闭页面
          if (ret.status) {
            this.unfull();
          } else {
            this.close();
          }
        });
      },
    );

    // 应用进入后台事件
    api.addEventListener(
      {
        name: 'pause',
      },
      () => this.pause(),
    );

    // 小窗口返回按钮点击
    this.module.addEventListener({
        name: 'backBtn',
      },
      (ret, err) => {
        this.close();
      },
    );

    // ios 右滑返回
    this.module.addEventListener({
        name: 'viewappear',
      },
      (ret, err) => {
        this.close();
      },
    );

    // 播放
    this.module.addEventListener(
      {
        name: 'play',
      },
      (ret, err) => {
        this.startReportPosition();
      },
    );

    // 暂停
    this.module.addEventListener(
      {
        name: 'pause',
      },
      (ret, err) => {
        this.stopReportPosition();
      },
    );

    // 进入全屏
    this.module.addEventListener(
      {
        name: 'full',
      },
      () => {
        api.sendEvent({
          name:'fullView'
        })
      },
    );

    // 播放结束
    this.module.addEventListener(
      {
        name: 'playEnd',
      },
      (ret, err) => {
        this.stopReportPosition();
      },
    );

    // 播放异常
    this.module.addEventListener(
      {
        name: 'playError',
      },
      (ret, err) => {
        this.stopReportPosition();
      },
    );
  },
};
